/* Copyright 2005, 2007 Ricardo Baratto */
/* Copyright 2005, 2007 Columbia University */
/* This file is part of THINC.

    THINC is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    THINC is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with THINC.  If not, see <http://www.gnu.org/licenses/>.
*/
/* 
 * zsound_test_enc.c: test program for sound encoding library
 *
 * Authors: 
 * Ricardo A. Baratto - Network Computing Lab - Columbia University
 */

/* reads wav from stdin, encodes it, and writes to stdout */

/* set tabstop = 8 */
#include "thincCommon.h"
#include "thinc_zsound.h"
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>

#define NUM_CHANNELS	2L
#define RATE		44100L
#define PAGE_SIZE	4096
#define QUALITY		1.0

#define error(...)	fprintf(stderr, ## __VA_ARGS__)

#if 0
#define TZ_TEST_DO_DEBUG
#endif

#ifdef TZ_TEST_DO_DEBUG
#define debug(...)	fprintf(stderr, ## __VA_ARGS__)
#else
#define debug(...)
#endif

static int test_write(void *handle, const void *data, unsigned int len);

int main(int argc, char **argv)
{
	struct thinc_zsound *zs;
	char *buffer;
	int err;
	long total=0;

	debug("hi there, welcome to the test program...\n");

	buffer = malloc(PAGE_SIZE);
	zs = thinc_zsound_enc_init(T_ZSOUND_VORBIS, NUM_CHANNELS, RATE,
		QUALITY);
	if(!zs || !buffer) {
		error("no init 4 u\n");
		return EXIT_FAILURE;
	}

	debug("i'm all initialized and almost ready to go, let me prepare "
		"something first tho\n");

	thinc_zsound_enc_setcb(zs, test_write, NULL);

	err = thinc_zsound_enc_prepare(zs);
	if(err == T_ERROR) {
		error("can't prepare\n");
		return EXIT_FAILURE;
	}

	debug("k, now we ready, let's have some fun!\n");

	/* encode data */
	while(!feof(stdin)) {
		err = fread(buffer, 1, PAGE_SIZE, stdin);
		if(err == -1) {
			error("can't read\n");
			return EXIT_FAILURE;
		} else if(err == 0) {
			debug("no more data\n");
			continue;	/* feof() should take care of breaking
					   loop */
		}
		total += err;
		debug("%ld bytes\r", total);
		err = thinc_zsound_enc_encode(zs, buffer, err);
		if(err == T_ERROR) {
			error("can't encode\n");
			return EXIT_FAILURE;
		}
	}
	debug("seems we're out of data, let's finish things up\n");
	err = thinc_zsound_enc_finish(zs);
	if(err == T_ERROR) {
		error("can't finish\n");
		return EXIT_FAILURE;
	}

	thinc_zsound_enc_end(zs);

	debug("and that was that... hope you didn't die\n");
	return EXIT_SUCCESS;
}

static int test_write(void *handle, const void *data, unsigned int len)
{
	return fwrite(data, len, 1, stdout);
}
