/*
 * $Id: $
 * $Source: $
 *
 * thincRecordCommon.h: common record header for all modules
 *
 * Author:
 * Ricardo Baratto <ricardo@cs.columbia.edu>
 * Network Computing Lab - Columbia University
 */

/* set tabstop=8 */

#ifndef _THINC_RECORD_COMMON_H_
#define _THINC_RECORD_COMMON_H_

/* if you change, don't forget the / at the end. lives depend on it */
#define T_REC_DEFAULT_DIR	"/var/cache/thinc/"
#define T_REC_DEFAULT_DIR_LEN (sizeof(T_REC_DEFAULT_DIR) - 1)

/* magic number for THINC files 'THIN' */
#define T_REC_MAGIC		{'T', 'H', 'I', 'N'}
#define T_REC_MAGIC_SZ		4

#define T_REC_IDX_NAME		"record.idx"
#define T_REC_SHOT_EXT		".shot"
#define T_REC_SHOT_EXT_LEN	5
#define T_REC_CMD_EXT		".cmd"
#define T_REC_CMD_EXT_LEN	4

/* length of a cmd/screenshot filename */
#define T_REC_FILEID_LEN	20

struct thincRecIndexHeader {
	/* TODO: what should we put here?
	 * probably:
	 * - fb dimensions
	 * - color depth
	 * - lsb or msb
	 * - last known command and screenshot files
	 *
	 * no need for:
	 * - first files (just look at first record)
	 */
	int			raw_cache_bits:8;
	int			pad:8;
	int			dummy:16;
};

#define T_REC_INDEX_HEADER_SZ	4

/* a record of a screenshot for the index file */
struct thincRecShot {
	/* doesn't include file extension or dir */
	char			file[T_REC_FILEID_LEN];
	int			fp:32;
};

#define T_REC_SHOT_SZ		(T_REC_FILEID_LEN + 4)

/* a record of a command for the index file */
struct thincRecCmd {
	/* doesn't include file extension or dir */
	char			file[T_REC_FILEID_LEN];
	int			fp:32;
};

#define T_REC_CMD_SZ		(T_REC_FILEID_LEN + 4)

/* a record in the index file */
struct thincRecRecord {
	unsigned long long	time:64;
	struct thincRecShot 	shot;
	struct thincRecCmd	cmd;
};

#define T_REC_RECORD_SZ		(8 + T_REC_SHOT_SZ + T_REC_CMD_SZ)

/* a record in the command file */
struct thincRecCmdHeader {
	unsigned long long	time:64;
	int			size:32;
	int 			type:8;
	int 			flags:8;
	char 			pad[2];
};

#define T_REC_CMD_HEADER_SZ	14

/* record showing what the next command file is */
#define T_REC_CMD_TYPE_NEXTFILE	0x01
#define T_REC_CMD_NEXTFILE_SZ	T_REC_CMD_HEADER_SZ + T_REC_FILEID_LEN

/* a record in the screenshot file */
struct thincRecShotHeader {
	unsigned long long	time:64;
	int			size:32;
	int			width:16;
	int			height:16;
	int			bpp:16;
	int			info:16;
};

#define T_REC_SHOT_HEADER_SZ	20

/* some info flags */

/* record showing what the next screenshot file is */
#define T_REC_SHOT_TYPE_NEXTFILE	0x0001
#define T_REC_SHOT_NEXTFILE_SZ	T_REC_SHOT_HEADER_SZ + T_REC_FILEID_LEN

/* screenshot is compressed */
#define T_REC_SHOT_COMPRESSED		0x0002

/****************************************************************************/
/* 
 * definitions for commands which we use also on the network protocol but
 * need to be modified for recording
 */

/* structures for caching data */
/* T_REC_CACHE_DATA
 * Description: Wrapper for a cached command. Goes inside a normal command,
 * just like T_FB_CACHE_DATA, and tells the client that the data for
 * this command can be found in the command pointed to by the struct.
 * Cache id, filename and offset are enclosed in the message
 * Size: T_REC_CMD_SZ + 4 bytes
 * Data None
 */
typedef struct thinc_recCacheData_s {
	int 			cache_id:32;
	struct thincRecCmd	cmd;
} thinc_recCacheData;

#define T_REC_CACHE_DATA_SZ	(T_REC_CMD_SZ + 4)

/* T_REC_ADD_CACHE:
 * Description: The update where this command is should be added to the
 * cache. The id to use is passed in this message
 * Size: 4 bytes
 * Data: None
 */
typedef struct thinc_recCacheAdd_s {
	int 			cache_id:32;
} thinc_recCacheAdd;

#define T_REC_CACHE_ADD_SZ	4

/* T_REC_VIDEO_NEXT
 * Description: same as T_VIDEO_NEXT except it has all information to make
 * it stateless. basically T_VIDEO_NEXT + T_VIDEO_START
 */
typedef struct thinc_recVidNext_s {
	int id		:32;	/* id for this video stream */
	int size	:32;	/* size of video data */
	int timestamp	:32;	/* timestamp for synchronization */
	/* extra info in case playback app didn't see START message */
	/* format id */
	int fmt_id	:32;	/* format. Only send id, which should have
				   been set on initialization */
	/* position */
	int x		:16;
	int y		:16;
	/* dimensions */
	int width 	:16;
	int height 	:16;
	/* for client scaling */
	int dst_width	:16;
	int dst_height  :16;
	/* followed by struct with format specific info (possible list
	 * below) of specified size */
	int fmt_size    :16;
	/*
	struct thinc_vidInfoYV12 yv12;
	struct thinc_vidInfoYUY2 yuy2;
	struct thinc_vidInfoUYVY uyvy;
	*/
	/* followed by video data */
} thinc_recVidNext;

#define T_REC_VIDEO_NEXT_SIZE	30

#endif /* _THINC_RECORD_COMMON_H_ */
