#!/usr/bin/perl -w
############################################################################
# Hack to parse server's log and create a list of transactions (CopyArea's)
# made.
#
# A transaction is defined as:
# SRC drawable COPIED to DST drawable a region of size SIZE
# (SIZE is shown as widthxheight)
# 
# Whose Fault is it:
# Ricardo Baratto - Network Computing Lab - Columbia University
# 
# TODO: Maybe we should print the complete region (x,y,w,h) instead
# of just (w,h)
#
# Released to the public domain. You may use this
# code any way you wish, private, educational, or commercial.  It's free.
############################################################################
#
### set tabstop 4 ###

use strict;
use Getopt::Std;

use vars qw(
	$ST_INIT 
	$ST_SRC 
	$ST_DST 
	$ST_AREA
	$ST_END 
);

# command line options
use vars qw(
	$opt_v
	$opt_o
);

getopts('vo');

main(@ARGV);

sub main {
	my %trans = ();
	my $curr = "";

	$ST_INIT = $ST_SRC = 0;
	$ST_DST = 1;
	$ST_AREA = 2;
	$ST_END = 3;

	my $state = $ST_INIT;

	my $infile = shift || die ("usage: $0 infile\n");

	open(INFILE, $infile) || die ("Can't open input file $infile: $!\n");

	while(<INFILE>) {
		chomp;

		# remove unnecessary stuff
		s/^\(II\)\s//;
		s/^\(EE\)\s//;
		s/^\(WW\)\s//;
		s/\((\d){2}:(\d){2}:(\d){2}\)\s//;

		if(/CopyArea/) {
			decide: {
				if ($state == $ST_INIT) {
					# look for SRC
					if(/SRC/) {
						# check what kind of drawable it is
						if(/Window/) {
							$curr = "W";
						} elsif(/Pixmap/) {
							$curr = "P";
						} else {
							die "agghhh... what is this drawable??? $_\n";
						}
						# remove things until we're left with the src id
						s/thincCopyArea: SRC://;
						s/\. Drawable.*//;
						s/id://;
						s/ //g;
						$curr .= " " . $_;
						$state = $ST_DST;
					}
					last decide;
				}
				if( $state == $ST_DST) {
					# look for DST
					if(/DST/) {
						# check what kind of drawable it is
						if(/Window/) {
							$curr .= " W";
						} elsif(/Pixmap/) {
							$curr .= " P";
						} else {
							die "agghhh... what is this drawable??? $_\n";
						}
						# remove things until we're left with the dst id
						s/thincCopyArea: DST://;
						s/\. Drawable.*//;
						s/id://;
						s/ //g;
						$curr .= " " . $_;
						$state = $ST_AREA;
					}
					last decide;
				}
				if( $state == $ST_AREA) {
					# look for size
					if(/size/) {
						# remove things until we're left with the size of
						# the copy
						s/thincCopyArea://;
						s/alu.*//;
						s/src (.*),//;
						s/[\.()]//g;
						s/size//;
						s/ //g;
						$curr .= " " . $_;
						$state = $ST_END;
					}
					last decide;
				}
				if( $state == $ST_END) {
					if(/finished/) {
						my $o = 0;
						if(/obscured/) {
							# didn't really take place
							$curr = "O " . $curr;
							$o = 1;
						}
						if($opt_o) {
							if($o) {
								print "ignoring obscured transaction $curr\n";
								$state = $ST_INIT;
								last decide;
							}
						}
						if(! $trans{$curr}) {
							$trans{$curr} = 1;
							if($opt_v) {
								print "Got new transaction: $curr\n";
							}
						} else {
							$trans{$curr}++;
							if($opt_v) {
								print "Got repeated transaction $curr\n";
							}
						}
						$state = $ST_INIT;
					}
					last decide;
				}
				die ("I'm lost!! What is this?? Moooooom!!! ($state)\n");
			}
		}
	}

	close(INFILE);

	my $ans;
	my $count = scalar(keys(%trans));
	print "\nGot $count transactions\n";
	print "Do you want to see them? [y/N] ";
	read (STDIN, $ans, 1);
	if($ans eq "y") {
		my $i = 1;
		my $k;
		foreach $k (keys (%trans)) {
			print "$i: $k => $trans{$k}\n";
			$i++;
		}
	}

	print "\ndat's it\n";
}
