#!/bin/bash

# set ts=4

# very crude script to create a release of thinc. run it
# from the directory where you want to put the release
# files.
# set RELEASE to the appropriate value, and if you want to
# change the directory for distribution, change TOP too.
# the rest should be fine.
#
# it will create tar.gz and tar.bz2 files. if you pass the
# -c switch, it will cleanup the release directory (so
# all you're left with is the tar.gz and tar.bz2 files)
#
# Released to the public domain. You may use this
# code any way you wish, private, educational, or commercial.  It's free.

PROGNAME="release.sh"
RELEASE="0.3.6"
DIRNAME="thinc"

#### shouldn't need to change anything below this #####

# parse options. from getopt examples (util-linux docs)
OPTIONS="chr:d:"
LONG_OPTIONS="clean,help,release:,dirname:"

usage() {
	echo "Usage: $PROGNAME [options]" >&2; 
	echo "options:" >&2;
	echo "        -h|--help        print this message" >&2;
	echo "        -c|--clean       clean all temporary files and dirs" >&2;
	echo "        -r|--release     specify the release (default is $RELEASE)" >&2;
	echo "        -d|--dirname     specify basename for release directory " >&2;
	echo "                         (default is $DIRNAME)" >&2;
}

TEMP=`getopt -o $OPTIONS --long $LONG_OPTIONS -n $PROGNAME -- "$@"`

if [ $? != 0 ]; then 
	usage; exit 1; 
fi

do_clean=0

eval set -- "$TEMP"

while true; do
	case "$1" in
		-c|--clean) 
			do_clean=1; 
			shift ;;
		-h|--help)
			usage;
			exit 0; 
			;;
		-r|--release)
			RELEASE=$2; 
			shift 2 ;;
		-d|--dirname)
			DIRNAME=$2;
			shift 2;;
		--) shift; break;;
		*) echo "Internal error"; exit 1;;
	esac
done

TOP=$DIRNAME-$RELEASE

echo "+ deleting old files"
rm -rf $TOP $TOP.tar.gz $TOP.tar.bz2

echo "+ exporting tree"
echo "++ common..."
tla export thinc-common--bloody--0.2 $TOP
echo "++ clients..."
cd $TOP
tla export thinc-clients--bloody--0.2 clients
echo "++ sound..."
tla export thinc-sound--bloody--0.1 sound
echo "++ server..."
mkdir server
cd server
tla export thinc-server--bloody--0.2 thinc
cd ..

# get out of top
cd ..

echo "+ writing out release info"
echo -e "This is version $RELEASE of thinc" >> $TOP/README
echo -e "The following trees were used in this release:\n" >> $TOP/README
tla revisions -f thinc-common--bloody--0.2 | tail -1 >> $TOP/README
tla revisions -f thinc-clients--bloody--0.2 | tail -1 >> $TOP/README
tla revisions -f thinc-sound--bloody--0.1 | tail -1 >> $TOP/README
tla revisions -f thinc-server--bloody--0.2 | tail -1 >> $TOP/README

echo "+ fixing up ownership"
chgrp -R thinc $TOP

echo "+ creating tarballs"
tar zcf $TOP.tar.gz $TOP
tar jcf $TOP.tar.bz2 $TOP

if [ $do_clean -eq 1 ]; then
		echo "+ cleaning up"
		rm -rf $TOP
fi

echo "done."
