#!/usr/bin/perl
#
# Hack to parse the thinc client's output and draw an image with nitfy colored 
# rectangles representing the different messages that the client received.
# Try it, it's cool!
#
# Authors:
# Ricardo Baratto - Network Computing Lab - Columbia University
#
# usage:
# client-graph.pl logfile imagefile
#
# Released to the public domain. You may use this
# code any way you wish, private, educational, or commercial.  It's free.
#
# set tabstop=4 #

use strict;
use GD;
use Getopt::Std;

my ($infile, $outfile, $black, $white, $count);
my ($x1, $y1, $x2, $y2, $color);
my ($png_data, $im);

$infile = shift || die("usage: $0 infile outfile $!\n");
$outfile = shift || die("usage: $0 infile outfile $!\n");

open(INFILE, $infile) || die "can't open input file $infile: $!\n";

# find out the size of the fb. in a very hackish way
my ($w, $h);
$w = $h = 0;
while(<INFILE>) {
	chomp;

	if(/Screen dimensions/) {
		s/.*Screen dimensions: //;

		($w, $h) = split(x);
		last;
	}
}

# went through the file and couldn't figure out fb size. not good
if($w == 0 or $h == 0) {
	die "i don't know the size of the framebuffer dude!\n";
}

print "framebuffer is $w x $h\n";
# new image
$im = new GD::Image($w,$h);

# allocate some colors
$black = $im->colorAllocate(0,0,0);
$white = $im->colorAllocate(255,255,255);

$im->rectangle(0,0,$w,$h,$white);

$count = 0;

my %colors = {};
my ($wavg, $havg);
$wavg = $havg = 0;

while(<INFILE>) {
	if(!/SFILL/) {
		next;
	}

	chomp;

	# strip off useless text
	s/^\(II\)\s//;
	s/^\(EE\)\s//;
	s/^\(WW\)\s//;

	s/\((\d){2}:(\d){2}:(\d){2}\)\s//;

	s/.*SFILL: //;
	($color, $x1,$y1,$x2,$y2) = split;

	if($colors{$color}) { $colors{$color}++; }
	else { $colors{$color} = 1; }

	$im->rectangle($x1,$y1,$x2,$y2, $white);
	++$count;
	$wavg += ($x2-$x1);
	$havg += ($y2-$y1);
}

print ("draw $count rectangles\n");

printf "average width %.2f, height %.2f\n", $wavg/$count, $havg/$count;

close(INFILE);

# save the image
open(OUTFILE, ">$outfile") || 
	die ("can't open output file $outfile: $!\n");

binmode OUTFILE;

$png_data = $im->png;

print OUTFILE $png_data;

close(OUTFILE);

my ($i, $cc);
$count = 0;
foreach $i (sort keys %colors) {
	$cc++;
	printf "color $i sawn %d times\n", $colors{$i};
	$count += $colors{$i};
}

print "$cc colors, total count $count\n";

print "done.\n";
