#!/usr/bin/perl -w
####################################################################
# Summary: Dirty yet handy script for parsing the THINC log file.
# Date created: 3/7/2003
#
# Author(s): 
# Leo Kim - Network Computing Lab - Computing University
# Ricardo A. Baratto - Network Computing Lab - Computing University
#
# Released to the public domain. You may use this
# code any way you wish, private, educational, or commercial.  It's free.
####################################################################
#
## set tabstop 4 ##

use strict;
use Getopt::Std;

# declare global variables
use vars qw(
    $DEFAULT_LOG_FILE 
    $DEFAULT_OUT_FILE 
    $DEFAULT_OUT_FILE_SUFFIX 
    $INDENT_LEVEL
    $END_OF_FILE_HEAD
    @FUNCTION_END_SIGNIFIERS
	%stats
);

# for option parsing
use vars qw(
    $opt_v
    $opt_f
    $opt_h
	$opt_s
	$opt_e
	$opt_q
	$opt_o
);


$END_OF_FILE_HEAD = "SaveScreen()";
$DEFAULT_OUT_FILE_SUFFIX = ".parsed";
$DEFAULT_LOG_FILE = "/var/log/XFree86.0.log";
$DEFAULT_OUT_FILE = "${DEFAULT_LOG_FILE}${DEFAULT_OUT_FILE_SUFFIX}";
@FUNCTION_END_SIGNIFIERS = ("finished", "not sending update", "error", "bug");
$INDENT_LEVEL = 1;

# parse opts
getopts('oqesvhf:');

# call main sub
main();

# auto indenter. set to two spaces.
sub doIndent {
    my $indentLevel = shift;
    my $i;
    for ($i = 0; $i < $indentLevel; $i++) {
        print "  ";
    }
}

# returns true if a keyword is found in the @FUNCTION_END_SIGNIFIERS array
# false if not.
sub isAtFunctionEnd {
    my $pattern;
    foreach $pattern (@FUNCTION_END_SIGNIFIERS) {
#        print "$pattern\n";
        if (/$pattern/i) {
            return 1;
        }
    }
    return 0;
}

sub hashStatsSort { $stats{$a} <=> $stats{$b}; }

sub printStats {
	my ($count, $k, $v, $i);

	print "\n\n=====================================\n";
	print "            Statitstics\n";
	print "=====================================\n\n";


	$count = scalar(keys(%stats)); 
	print "Saw $count functions\n\n";

	$i = 1;
	# print stats sorted
	if($opt_o) {
		foreach $k (sort hashStatsSort (keys (%stats))) {
			print "$i. $k => $stats{$k}\n";
			$i++;
		}
	} else {
		while(($k,$v) = each %stats) {
			print "$i. $k => $v\n";
			$i++;
		}
	}
}

# prints program usage
sub Usage {
	print STDERR "\nUsage: $0 <options> [-f <filename>]\n\n";
	print STDERR "-f:        specifies log file to parse. If ommited \n";
	print STDERR "           /var/log/XFree86.0.log is used\n\n";
	print STDERR "options:\n";
	print STDERR "-v:        be verbose\n";
	print STDERR "-h:        print this message\n";
	print STDERR "-e:        Don't look for \"SaveScreen\" before starting\n";
	print STDERR "           to parse (SaveScreen is point where\n";
	print STDERR "           interesting things start to happen in a \n";
	print STDERR "           nornal log\n";
	print STDERR "-q:        Quiet. Doesn't print the functions as they're\n";
	print STDERR "           found. Note that -q has higher precendence\n";
	print STDERR "           than -v\n";
	print STDERR "-s:        Print statistics on the functions found\n";
	print STDERR "-o:        Print statistics sorted\n";
	exit(0);
}

# helper for sorting the statistics
# a and b are the arguments that sort passes to the function
sub main {
	# see if just want help
    if ($opt_h) {
		Usage();
	}

	if($opt_q) {
		$opt_v = 0;
	}
    if (!$opt_f) {
		# since we normally just want to use this file... -ricardo
		print STDERR "\nUsing default log file $DEFAULT_LOG_FILE\n";
		$opt_f = $DEFAULT_LOG_FILE;
    }

	if($opt_o and !$opt_s) {
		$opt_o = 0;
	}
    my $indent = 0;
    my @stack;
    my @words,
    my $functName = "";
	%stats = ();

    open (INFILE, $opt_f)               or die "Can't open $opt_f: $!";
#    open(INFILE, $DEFAULT_LOG_FILE)     or die "Can't open $DEFAULT_LOG_FILE: $!";
#    open(OUTFILE, ">$DEFAULT_OUT_FILE") or die "Can't open $DEFAULT_OUT_FILE: $!";

    # read until SaveScreen is encountered. This is where the interesting
    # stuff starts happening
	# TODO Seems like a good signature for things we know how to parse is
	# /thinc[A-Z].*:/, so instead of all this SaveScreen mess we could just
	# do:
	# if(not /thinc[A-Z].*:/) {
	# 	next;
	# }
	if(!$opt_e) {
		while (<INFILE>) {
			if (/$END_OF_FILE_HEAD/) {
				last;
			}
		}
	}

    # read file line by line
    while (<INFILE>) {
		# Check we're not at the end of the parsable output -ricardo
		if(/CloseScreen/i) {
			last;
		}

		# ignore things we don't know how to parse -ricardo
		if(/THINCDEV/i) {
			next;
		}

#       print OUTFILE $_;
        # algorithm: 
        # - when a function name is read, push it on the stack and indent.
        # - while function is on stack, ignore all other instances of function name
        #   (but print out names of other functions).
        # - once "finished" signifier is read, pop stack and decrease indent.

        # strip off text that looks like this: "(II) (EE) (WW) "
        s/^\(II\)\s//;
        s/^\(EE\)\s//;
        s/^\(WW\)\s//;
        
        # strip off time stamp of form "(hh:mm:ss)". maybe make this optional
        # in the future or group with previous regular expression.
        s/\((\d){2}:(\d){2}:(\d){2}\)\s//;

		# get the first word in the line
		@words = split (/[:\s+]/);
		$functName = $words[0];

		# replace periods and colons
		$functName =~ s/[:.]//;

        # if a function-ending signifier is found, then the stack is popped and
        # the indent is decreased
        if (isAtFunctionEnd()) {
            $indent--;
			# be more resilient to non-correct output. If the word on top
			# is not the one we were supposed to match keep popping until we
			# find one that matches. -ricardo
            my $top = pop(@stack);
			while(($#stack >=0) and ($top ne $functName)) {
				$top = pop(@stack);
				if($opt_v) {
					print STDERR "non-matching function at top $top\n";
				}
            	$indent--;
			}
			if(!$opt_q) {
				doIndent($indent);
			}
			if($opt_v) {
	            print "$_";
			}
        } else {
			# added chk for stack non-empty so -w wouldn't complain -ricardo
            if (($#stack < 0) or ($stack[$#stack] ne $functName)) {
                # push function name on to stack
                push(@stack, $functName);
				if(! $stats{$functName}) {
					# new word
					$stats{$functName} = 1;
				} else {
					$stats{$functName}++;
				}
				if(!$opt_q) {
					doIndent($indent);
				}
				
				if ($opt_v) {
                    print "$_";
                } elsif(!$opt_q) {
                    print "$functName\n";
                }
                $indent++;
            }
        } # end isAtFunctionEnd()
    } # end while(<INFILE>)

    # don't forget to close the open file handles!
    close INFILE;

	if($opt_s) {
		printStats();
	} 
}

